# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import json

import requests

from .base import BaseChannel
from channels.exceptions import HttpError, ImproperlyConfigured


class SlackChannel(BaseChannel):
    def __init__(self, url, username=None, channel=None, icon_emoji=None,
                 icon_url=None, *args, **kwargs):
        self.url = url
        self.username = username
        self.channel = channel
        self.icon_emoji = icon_emoji
        self.icon_url = icon_url

        # Validation
        if self.icon_emoji is not None and self.icon_url is not None:
            raise ImproperlyConfigured(
                "Must not set both icon_emoji and icon_url")

    def send(self, message, fail_silently=False, options=None):
        payload = {
            "text": message
        }
        if self.channel is not None:
            payload["channel"] = self.channel
        if self.username is not None:
            payload["username"] = self.username
        if self.icon_emoji is not None:
            payload["icon_emoji"] = self.icon_emoji
        if self.icon_url is not None:
            payload["icon_url"] = self.icon_url

        self._set_payload_from_options(payload, options, "slack", [
            "attachments", "unfurl_links"])

        try:
            response = requests.post(self.url, data={
                "payload": json.dumps(payload)
            })
            if response.status_code != requests.codes.ok:
                raise HttpError(response.status_code, response.text)
        except:
            if not fail_silently:
                raise



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import json

import requests

from .base import BaseChannel
from channels.exceptions import HttpError, ImproperlyConfigured


class HipChatChannel(BaseChannel):
    colors = ("yellow", "green", "red", "purple", "gray", "random")
    message_formats = ("html", "text")

    def __init__(self, api_id, token, base_url="https://api.hipchat.com/v2/",
                 color=None, notify=None, message_format="html", *args,
                 **kwargs):
        self.url = "{0}room/{1}/notification?auth_token={2}".format(
            base_url, api_id, token)
        self.color = color
        self.notify = notify
        self.message_format = message_format

        # Validation
        if self.color is not None:
            if self.color not in self.colors:
                raise ImproperlyConfigured("Invalid color")

        if self.notify is not None:
            if not isinstance(self.notify, bool):
                raise ImproperlyConfigured("Notify must be bool")

        if self.message_format not in self.message_formats:
            raise ImproperlyConfigured("Invalid message_format")

    def send(self, message, fail_silently=False, options=None):
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        payload = {
            "message": message,
            "message_format": self.message_format
        }

        if self.color is not None:
            payload["color"] = self.color
        if self.notify is not None:
            payload["notify"] = self.notify

        self._set_payload_from_options(payload, options, "hipchat", [
            "color", "notify", "message_format"])

        try:
            response = requests.post(self.url,
                                     headers=headers,
                                     data=json.dumps(payload))
            if response.status_code != requests.codes.no_content:
                raise HttpError(response.status_code, response.text)
        except:
            if not fail_silently:
                raise



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
import sys

from .exceptions import ImproperlyConfigured


_BACKENDS = {}


def _load_module(name):
    __import__(name)
    return sys.modules[name]


def _load_backend(name):
    try:
        return _BACKENDS[name]
    except KeyError:
        module_name, klass_name = name.rsplit(".", 1)
        module = _load_module(module_name)
        _BACKENDS[name] = getattr(module, klass_name)
        return _BACKENDS[name]


def _load_django_settings():
    from django.conf import settings
    return settings.CHANNELS


def _send(message, settings, channel_name, fail_silently, options):
    if channel_name is None:
        channels = settings["CHANNELS"]
    else:
        try:
            channels = {"channel": settings["CHANNELS"][channel_name]}
        except KeyError:
            raise Exception("channels does not exist %s", channel_name)

    for _, config in channels.items():
        if "_backend" not in config:
            raise ImproperlyConfigured(
                "Specify the backend class in the channel configuration")

        channel = _load_backend(config["_backend"])(**config)
        channel.send(message, fail_silently=fail_silently, options=options)


def send(message, channel=None, fail_silently=False, options=None):
    """Send a notification to channels

    :param message: A message
    :type message: str | unicode
    :type fail_silently: bool
    :type options: dict
    """
    settings = _load_django_settings()
    _send(message, settings, channel, fail_silently, options)
